/****************************************************************
*	Magics.Com - Update the magic filenames alias files	*
*****************************************************************/
/*	Bill Birrell (c) 1990 - PD Release 1994 		*/
/*	MicroSoft C Source code 				*/
/****************************************************************/
/*  History							*/
/* version # 1.01						*/
/* Modification - get rid of spurious nonsense filenames	*/
/* version # 1.02						*/
/* make a log file to record the date and time the prog is run	*/
/* create syspath\magics.log and write asctime to it		*/
/* Version # 1.03						*/
/* make dofile() to avoid repeating the same code in main	*/
/* Version # 1.04						*/
/* modify program to read a control file if there is one	*/
/* version # 1.05						*/
/* fix uninitialised variable					*/
/* version # 1.06						*/
/* Move array majics to a static to avoid memory crash		*/
/* version # 1.07						*/
/* Now include the ability to insert comments in the .ctl file	*/
/* and the alias files. Deal properly with the case of 0 format */
/* clean up the process() function. Fix the syntax option.	*/
/* clean up the readfile function.				*/
/* version # 1.08 [(1.09) remove typing errors]                 */
/* Tricky - if there is a [/-]digit command line parameter, the */
/* call to getenv(FD) does not take place, and the syspath      */
/* variable is set to "."   This allows multi-USERNO versions of*/
/* FrontDoor to look up the local setup.fd for data, and to use */
/* the FD%d.LOG file for recording updates.                     */
/****************************************************************/

#include <stdio.h>
#include <dos.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <io.h>
#include <time.h>
#define word unsigned
#define byte unsigned char
#define dword long
#pragma pack(1)
#include "ctl.h"

char version[]="Version 1.10";

/* Globals */
static char buffer[128];
char mainfl[2][71], tmpfl[2][71], bakfl[2][71];
char token[14], latest[16], dirname[32], newdir[32],newfile[14];
char drive[3],dir[32],name[14],ext[5],name1[14],ext1[5],nodrive[3],nodir[32];
char sysdir[64], logfile[64];
char errmsg[32], pllog[32], pwlog[32];
char sheesh[32];
static int QUIET, HELP, DEBUG, NTOKENS, GLOBAL, MULTI, USERNO, f[2];
FILE *fp1, *fp2, *fp3, *fp4;
struct params {
    char token[10];
    char format[6];
};
static struct find_t find_buffer;
static struct _ctl setup_buf;
static struct params majics[64];

/* functions */

void errexit(int a, char *s)
{
    fprintf(stderr,"%s\n",s); exit(a);
}

void option(char *g)
{
    if(*g == '/' || *g == '-') {
        ++g;
	switch(toupper(*g)) {
            case 'G'        : GLOBAL = 1;
			      break;
            case 'Q'        : QUIET = 1;
			      break;
	    case '1' :
	    case '2' :
	    case '3' :
	    case '4' :
	    case '5' :
	    case '6' :
	    case '7' :
	    case '8'        : MULTI = 1;
			      USERNO=(int) (*g - '0');
			      break;
	    default         : HELP = 1;
			      break;
            }
    }
}

int readfile(char *s)
{
    int ntokens=0;
    if((fp2 = fopen(s, "rb")) == NULL) {
        fclose(fp2);
	printf("error: can't open %s\n",s);
        exit(-1);
        };
    while(fgets(buffer, sizeof(buffer), fp2)) {
	if(!isalpha(*buffer)) continue;
	sscanf(buffer, "%s %s", majics[ntokens].token,majics[ntokens].format);
        ntokens++;
        }
    fclose(fp2);
    return ntokens;
}

int do_args(int argc, char *argv[])
{
    int i=0;

    if(argc != 1)
        while(--argc) {
        ++argv;
        switch (**argv) {
            case '/'        :
            case '-'        : option(*argv);
			      break;
	    default         : i=readfile(*argv);
			      break;
            };
        };
	return i;
}

int logit (int a, int b)
{
    time_t ltime;

    if((fp4=fopen(logfile,"a+")) == NULL)
		errexit(-6,"magics: can't open log file");
    if(!MULTI)
    {
	if(a) {
	       fprintf(fp4,"%s ",pwlog);
	       if(b) fprintf(fp4,"& %s ",pllog);
	       }
	else if (b) fprintf(fp4,"%s ",pllog);
	fprintf(fp4,"updated. ");
	time(&ltime);
	fprintf(fp4,"%s",ctime(&ltime));
    }
    else
    {
	time(&ltime);
	if (a) {
		sprintf(sheesh,"%s",ctime(&ltime));
		sheesh[strlen(sheesh)-6]='\0';
		fprintf(fp4,"+ %s %s ",&sheesh[11],pwlog);
		if(b) fprintf(fp4,"& %s ",pllog);
		}
	else if(b) fprintf(fp4,"+ %s %s ",ctime(&ltime),pllog);
	fprintf(fp4,"updated.\n");
    }
    fclose(fp4);
    return 0;
}

void signon(void)
{
	printf("\nMagics %s - Copyright (C) 1990 Bill Birrell\n",version);
	return;
}

char *mixcase(char *thisname)
{       register int n = -1, change=1;
	auto int m = strlen(thisname);
	/********************************************/
	/* prettify the output - else it's all caps */
	/********************************************/
	while( ++n < m ) {
		if(change) thisname[n]=(char)toupper(thisname[n]);
		else thisname[n]=(char)tolower(thisname[n]);
		if(strchr(":\\.",thisname[n])) change=1;
		else change=0;
		}
	return thisname;
}

char *search_for(char *filename)
{
	/* find the latest versions of the files */
	unsigned n, m, o, p;
	m=n=o=p=0;
	strcpy(latest,"");
	_dos_findfirst(filename,_A_NORMAL,&find_buffer);
	n=find_buffer.wr_date; o=find_buffer.wr_time;
	strcpy(latest,find_buffer.name);
	while(_dos_findnext(&find_buffer) == 0)
	{
		m=find_buffer.wr_date;
		p=find_buffer.wr_time;
		if(m > n)
		{
			n=m;
			strcpy(latest,find_buffer.name);
		}
		else if(m == n)
		{
			if(p > o)
			{       n=m;
				o=p;
				strcpy(latest, find_buffer.name);
			}
		}
	};
	return(latest);
}

void process(FILE *fp)
{       char tmp[9];
	extern char token[], dirname[];
	int j, n;
	while(fgets(buffer,100,fp) != NULL)
	{
		sscanf(buffer,"%s\t%s",token,dirname);
		if ( !isalpha( *buffer)) continue;
		_splitpath(dirname,drive,dir,name,ext);
		strcpy(name1, name);
		strcpy(ext1,ext);
		j=6;			/* set up default name length*/
		strcpy(ext,"*");	/* and default ext */
		if(NTOKENS)	/* if there a config file */
		   for (n = 0; n < NTOKENS; n++)
		      if(strcmp(strupr(token),strupr(majics[n].token)) == 0)
			{
				j = *majics[n].format & 017;
				if( j == 0) break;
				strcpy(ext,strupr(&majics[n].format[2]));
				break;
			};
		if(j) {
		    strncpy(tmp,name,j);
		    for(n=j; n < 8; n++) tmp[n]='?';
		    tmp[8]='\0';
		    strcpy(name,tmp);
		    _makepath(newdir,drive,dir,name,ext);
		    strcpy(newfile, search_for(newdir));
		    _splitpath(newfile,nodrive,nodir,name,ext);
		    if(strncmp(strupr(name1),strupr(name),j) == 0)
			_makepath(newfile,drive,dir,name,ext);
		    }
		else _makepath(newfile,drive,dir,name1,ext1);
		strcpy(newfile,mixcase(newfile));
		sprintf(buffer,"%- 8s\t%s\n",token,newfile);
		if(QUIET == 0) printf("%s",buffer);
		fputs(buffer,fp3);
	}
	fclose(fp);
	fclose(fp3);
	return;
}

int dofile(int g)
{
	if (( fp1=fopen(mainfl[g],"r")) != NULL) {
        fp3=fopen(tmpfl[g],"w");
        if(fp3 == (FILE *) NULL) {
            fclose(fp1);
            sprintf(errmsg,"magics: can't open %s",tmpfl[g]);
            errexit(-3, errmsg);
            };
        f[g]=1;
        if (!QUIET) printf("\nUpdating file %s :-\n",mainfl[g]);
        process(fp1);

        unlink(bakfl[g]);
        rename(mainfl[g],bakfl[g]);
        rename(tmpfl[g],mainfl[g]);
        };
        return 0;
}

int makefiles(int t)
{
    _splitpath( mainfl[t],  drive, dir, name, ext);
    _makepath(  tmpfl[t],   drive, dir, name, "$$$");
    _makepath(  bakfl[t],   drive, dir, name, "BAK");
    return 0;
}

int getsetup (void)
{
    int n, fd;

    if(sysdir[strlen(sysdir)]!='\\') strcat(sysdir,"\\");
    strcpy(logfile,sysdir);
    strcat(sysdir,"setup.fd");
    if(!MULTI) strcat(logfile,"magics.log");
    else {      sprintf(sheesh,"FD%d.LOG",USERNO);
		strcat(logfile,sheesh);
		}
    fd=open(sysdir,(int) (O_RDONLY | O_BINARY));
    if(fd == -1) errexit(-2,"magics: can't open setup.fd");
    n = read(fd,(char *)&setup_buf, (unsigned) sizeof(struct _ctl));
    close(fd);
    if (n != sizeof(struct _ctl)) errexit(-5,"magics: read error on Setup.Fd");
    strcpy( mainfl[0], setup_buf.m.reqsecalias);
    strcpy( pwlog,	  setup_buf.m.reqsecalias);
    strcpy( mainfl[1], setup_buf.m.reqalias);
    strcpy( pllog, setup_buf.m.reqalias);
    makefiles(0);
    makefiles(1);
    return 0;
}

void syntax(void)
{
    printf("Magics { [/g /h /q] filename.ext }\n");
    printf("          /h print this message\n");
    printf("          /q suppress screen output\n");
    printf("filename.ext is an optional configuration file\n");
    exit(0);
}

int  main(int argc, char **argv)
{
    char *s;
    int n=0;
    extern void signon(void), syntax(void);

    NTOKENS = do_args(argc,argv);
    if(HELP) syntax();
    if(!QUIET || DEBUG) signon();
    if(DEBUG) for(n = 0; n < NTOKENS; n++) {
	printf( "%- 8s\t%- 4s\n", majics[n].token, majics[n].format);
    };
    if(!MULTI) {
	if((s=getenv("FD"))==NULL) errexit(-1,"FD Environment variable not set\n");
	else strcpy(sysdir,s);
	}
    else strcpy(sysdir,".");
    getsetup();
    dofile(0);
    dofile(1);
    if(DEBUG) printf("\ngoing to logit\n");
    return logit(f[0],f[1]);
}
